import "Turbine.UI.Lotro"
import "MinstrelBuffs.MinstrelBuffWindow.Class";

InCombatTimerControl = class(TimerControl);

-- Design Notes:
-- 
-- LOTRO treats entering combat different based on if you were recently in combat
--   (your 9 second timer is still active) or not.
--
-- If you re-enter a combat state, you have to stay in it for 5 seconds
--   to actually restart the timer. However, if the timer expires during a too-short combat,
--   the timer does get refreshed.
--
-- Activating a ballad without being in combat (Resonance Major Ballad, Cry of the Chorus)
--   starts a new out of combat timer if one isn't running.
-- 

function InCombatTimerControl:Constructor()
    TimerControl.Constructor(self);
    self.minimumCombatTime = 5;
    self.CombatActivated = false;
    self.CryOfTheChorusActivated = false;
    self.CryOfTheChorusTime = nil;
    self.MajorBalladActivated = false;
    self.MajorBalladTime = nil;
    self.PrimaryCombat = false;

    -- If the plugin is loaded while combat is active, behave sensibly:
    if (LocalPlayer:IsInCombat()) then
        self:EnteringCombat();
    end
end

function InCombatTimerControl:CryOfTheChorus()
    -- Cry of the Chorus should start a timer if:
    --  a) the player is not already in combat
    --  b) there is not already a combat countdown happening
    --  c) there is not already a major ballad countdown happening
    --  d) there is not already a cry of the chorus countdown happening
    if (not LocalPlayer:IsInCombat() and not self.CombatActivated and not self.MajorBalladActivated and not self.CryOfTheChorusActivated) then
        self.CryOfTheChorusActivated = true;
        self.CryOfTheChorusTime = Turbine.Engine.GetGameTime();
        self:SetStartTime(Turbine.Engine.GetGameTime());
        self:SetWantsUpdates(true);
    end
end

function InCombatTimerControl:MajorBallad()
    -- Major Ballad (Resonance) should start a timer if:
    --  a) the player is not already in combat
    --  b) there is not already a combat countdown happening
    --  c) there is not already a major ballad countdown happening
    --  d) there is not already a cry of the chorus countdown happening
    if (not LocalPlayer:IsInCombat() and not self.CombatActivated and not self.MajorBalladActivated and not self.CryOfTheChorusActivated) then
        self.MajorBalladActivated = true;
        self.MajorBalladTime = Turbine.Engine.GetGameTime();
        self:SetStartTime(Turbine.Engine.GetGameTime());
        self:SetWantsUpdates(true);
    end
end

function InCombatTimerControl:EnteringCombat()
    self.enterCombatTime = Turbine.Engine.GetGameTime();

    -- This is a primary combat if:
    --  a) the combat timer is not currently running
    --  b) the major ballad timer is not currently running
    if (not self.CombatActivated and not self.MajorBalladActivated) then
        self.PrimaryCombat = true;
    end

    -- When first initialized, the width is 0 to account for being visible outside of combat. Fix that here:
    if (self:GetWidth() == 0) then
        self:SetWidth(self.TimerWidth);
    end

    -- either way we're going to want to process update messages:
    self:SetWantsUpdates(true);
end

function InCombatTimerControl:LeavingCombat()
    local leaveCombatTime = Turbine.Engine.GetGameTime();

    local isSecondaryCombatLongEnough = (leaveCombatTime - self.enterCombatTime) > self.minimumCombatTime;

    -- When leaving combat, the timer should reset if:
    --  a) this is a primary combat
    --  b) this is a secondary combat that lasted long enough
    --  c) this is a secondary combat that occured while the timer hit 0
    if (self.PrimaryCombat or isSecondaryCombatLongEnough or self.ranOverTime) then
        -- start the clock over again:
        self.CombatActivated = true;

        -- clear all three possible ways we could get here:
        self.PrimaryCombat = false;
        self:SetStartTime(leaveCombatTime);
        self.ranOverTime = false;
    end
end

function InCombatTimerControl:Update()
    local currentTime = Turbine.Engine.GetGameTime();

    -- Check some conditions:
    local hasCryTimedOut = self.CryOfTheChorusTime ~= nil and ((currentTime - self.CryOfTheChorusTime) >= 9);
    local shouldTurnOffCry = self.CryOfTheChorusActivated and hasCryTimedOut;

    local hasMajorBalladTimedOut = self.MajorBalladTime ~= nil and ((currentTime - self.MajorBalladTime) >= 9);
    local shouldTurnOffMajorBallad = self.MajorBalladActivated and hasMajorBalladTimedOut;

    local isPrimaryCombat = self.PrimaryCombat;
    local isSecondaryCombatLongEnough = self.enterCombatTime ~= nil and ((currentTime - self.enterCombatTime) > self.minimumCombatTime);
    local isQualifyingCombat = LocalPlayer:IsInCombat() and (isPrimaryCombat or isSecondaryCombatLongEnough);

    local didTimeRunOut = self:GetStartTime() ~= nil and (currentTime - self:GetStartTime() > self:GetDuration());
    local didTimeRunOutWithCombatActive = didTimeRunOut and LocalPlayer:IsInCombat();

    -- Will be set to true if the timer bar should stop shrinking:
    local pauseTimer = false;

    -- Determine what timer state (if any) is happening:

    -- Is a qualifying combat underway, or are we in combat at the end of the timer?:
    if (isQualifyingCombat or didTimeRunOutWithCombatActive) then
        -- Combat overrides other timer sources:
        self.CryOfTheChorusActivated = false;
        self.MajorBalladActivated = false;

        -- We're in a combat state now:
        self.CombatActivated = true;

        -- As long as we're in a qualifying combat state, restart the 9 second timer.
        self:SetStartTime(currentTime);

        -- Let other logic know we ran over:
        if (didTimeRunOutWithCombatActive) then
            self.ranOverTime = true;
        end
    -- Was Cry of the Chorus active and just run out?
    elseif (shouldTurnOffCry) then
        self.CryOfTheChorusActivated = false;
        self:SetWantsUpdates(false);
    -- Was Major Ballad active and just run out?
    elseif (shouldTurnOffMajorBallad) then
        self.MajorBalladActivated = false;
        self:SetWantsUpdates(false);
    -- Are we in a possibly-qualifying combat but need to wait a bit?
    elseif (LocalPlayer:IsInCombat()) then
        -- Pause the timer to show we're waiting on a longer combat.
        pauseTimer = true;
    elseif (didTimeRunOut) then
        -- The combat timer just ran out:
        self.CombatActivated = false;
        self:SetWantsUpdates(false);
    end

    -- The bar should be counting down when:
    --   a) counting down after Cry of the Chorus while out of combat
    --   b) counting down after Major Ballad (Resonance) while out of combat
    --   c) counting down after leaving a qualifying combat:
    --      1) a primary combat that starts when the timer is not active
    --      2) a secondary combat that starts while the timer is running that lasts at least 5 seconds,
    --          or past the end of the existing timer
    -- It should pause while waiting to confirm if a secondary combat is long enough.
    if (not pauseTimer) then
        local uiTimerWidth = math.max(0, self:GetRemainingPercent() * self.TimerWidth);
        self:SetWidth(uiTimerWidth);
    end

end
